@@ -11,13 +11,13 @@
 	projected = TRUE
 	for (i in seq(along = object$data)) {
 		d = object$data[[i]]
+		beta[[i]] = d$beta
 		if (i > 1 && !identical(proj4string(object$data[[1]]$data), proj4string(d$data)))
 			stop("data items in gstat object have different coordinate reference systems")
 		raw = gstat.formula(d$formula, d$data)
 		y[[i]] = raw$y
 		locations[[i]] = raw$locations
 		X[[i]] = raw$X
-		beta[[i]] = raw$beta
 		grid[[i]] = raw$grid
 		if (is(d$data, "Spatial"))
 			projected = is.projected(d$data)
